@@ -4,23 +4,26 @@ from __future__ import print_function
 from sys import version_info
 from datetime import datetime, timedelta
 from . import __version__ as version
+
 import argparse
-import logging
-import time
-import numpy as np
 import glob
 import hashlib
-import pandas
+import json
+import logging
+import numpy as np
 import os
+import pandas
 import pprint
 import re
+import shlex
 import strictyaml
 import subprocess
-import shlex
 import sys
+import time
+import traceback
+import urllib3
 import yaml
 
-
 version.get_more_info()
 
 #   > 1. Parser Setup
@@ -128,6 +131,29 @@ ILLUMINA_LEGAL = "Oligonucleotide sequences © 2020 Illumina, Inc."
 workdir_is_set = False
 config_is_set = False
 
+LEGACY_YAML_CONFIG_SCHEMA =  strictyaml.Map(
+    {"samples": strictyaml.Str(),
+     "output": strictyaml.Str(),
+     "reference": strictyaml.Str(),
+     "run_id": strictyaml.Str(),
+     "var_annotation": strictyaml.Bool(),
+     "cns_annotation": strictyaml.Str(), 
+     "pangolin": strictyaml.Str(),
+     "primer": strictyaml.Str(),
+     "adapter": strictyaml.Str(),
+     "var_special_interest": strictyaml.Str(),
+     "var_call_cov": strictyaml.Int(),
+     "var_call_count": strictyaml.Int(),
+     "var_call_frac": strictyaml.Float(),
+     "var_filter_mqm": strictyaml.Int(),
+     "var_filter_sap": strictyaml.Int(),
+     "var_filter_qual": strictyaml.Int(),
+     strictyaml.Optional("read_filter_len"): strictyaml.Int(),
+     strictyaml.Optional("read_filter_qual"): strictyaml.Int(),
+     "cns_min_cov": strictyaml.Int(),
+     "cns_gt_adjust": strictyaml.Float(),
+     "krakenDb": strictyaml.Str(),
+     "krakenTaxID": strictyaml.Int()})
 
 # Settings for Default Config Schema
 STRICT_YAML_CONFIG_SCHEMA = strictyaml.Map(
@@ -147,6 +173,8 @@ STRICT_YAML_CONFIG_SCHEMA = strictyaml.Map(
      "var_filter_mqm": strictyaml.Int(),
      "var_filter_sap": strictyaml.Int(),
      "var_filter_qual": strictyaml.Int(),
+     "read_filter_len": strictyaml.Int(),
+     "read_filter_qual": strictyaml.Int(),
      "cns_min_cov": strictyaml.Int(),
      "cns_gt_adjust": strictyaml.Float(),
      "krakenDb": strictyaml.Str(),
@@ -268,6 +296,7 @@ def get_snakemake_info():
     
 
 def main(snakemake=None,cmd=None):
+    supress_error = False
     if snakemake is not None:
         cmd = get_snakecmd(snakemake) # Snakemake Support in >Utility Section<
     version_string = ("Version: {vers} - {ahead} ({commit}) {branch}".format(
@@ -299,6 +328,14 @@ def main(snakemake=None,cmd=None):
                              logging.DEBUG 
                              if args.debug 
                              else logging.INFO)
+            newer_version = get_newer_version()
+            if newer_version is not None:
+                logger.warning("!!There is a newer version available!!\n"
+                               "You are using         {vers}\n"
+                               "The newest release is {new_vers}\n\n"
+                               "Install it with \"pip install covpipe --upgrade\""
+                               "".format(vers=version.VERSION, 
+                                         new_vers=newer_version))
             logger.info("\n"
                         "  ################################\n"
                         "  #                              #\n"
@@ -363,6 +400,7 @@ def main(snakemake=None,cmd=None):
             #logger.exception(e)
             logger.error("\n\nSnakemake Stdout Log:\n{elog}".format(elog=e.stdout))
             logger.error("\n\nSnakemake Error Log:\n{elog}".format(elog=e.stderr))
+            supress_error = True
             raise e
     except QUIT_SAFE_ERROR:
         logger.error("\nQuitting because program encountered Error")
@@ -383,10 +421,32 @@ def main(snakemake=None,cmd=None):
                "incoming+rkibioinformaticspipelines-ncov-minipipe-17578161-issue-@incoming.gitlab.com\n"
                )
 
-        logger.exception(e)
+        if not supress_error:
+            logger.exception(e)
         exit(2)
 
 
+def get_newer_version(proxy_par=None):
+    retval = None
+    url = "http://gitlab.com/api/v4/projects/17578161/releases"
+    try:
+        if proxy_par is None:
+            proxy_par = os.getenv("HTTP_PROXY")
+        if proxy_par is not None:
+            proxy = urllib3.ProxyManager(proxy_par)
+            r = proxy.request("GET", url)
+        else:
+            http = urllib3.PoolManager()
+            r = http.request("GET", url)
+        data = json.loads(r.data)
+        new_vers_string = data[0]["tag_name"].strip("v")
+        new_vers = tuple(new_vers_string.split("."))
+        if tuple(version.VERSION.strip("v").split(".")) < new_vers:  
+            retval = new_vers_string
+    except json.decoder.JSONDecodeError:
+        pass
+    return retval
+
 
 #   # 1. Parser Setup
 
@@ -492,6 +552,14 @@ def get_parser(version_string="Version: {vers} "
                         help="Specify a common directory to store conda environments."
                              " Default is using the value of the environment variable SNAKE_CONDA_PREFIX."
                              " If said env var is unset, use the current working directory")
+    readgroup = parser.add_argument_group("Read Filter Options")
+    readgroup.add_argument("--read_filter_len", default=50, type=int,
+                           help="Minimum Read length. "
+                                "Defaults to 50 BP "
+                                "and should be lowered when using very short read protocols")
+    readgroup.add_argument("--read_filter_qual", default=20, type=int,
+                           help="Quality threshold for qualified quality "
+                                 "of read as a Phredscore. Default: 20")
     vargroup = parser.add_argument_group("Variation Calling in Filter Options")
     vargroup.add_argument("--var_call_cov", default=20, type=int, 
                           help="Minimum coverage value to use for variant call."
@@ -991,6 +1059,7 @@ def build_config_from_args(args):
     if args.output:
         config_location=args.output
     schema = STRICT_YAML_CONFIG_SCHEMA
+    fallbackSchema = LEGACY_YAML_CONFIG_SCHEMA 
     found_config = False
     sample_conf_path = None
     config_file_path = None
@@ -999,13 +1068,19 @@ def build_config_from_args(args):
     with open(DEFAULT_CONFIG_TEMPLATE, "r") as conf_fh:
         try:
             config = strictyaml.load(conf_fh.read(), schema)
-        except strictyaml.exceptions.YAMLValidationError as e:
-            context = e.args[2]
-            logger.error("Error in Config:\n{before}\n"
-                         " #vvvv Error Line vvvv#{lines}".format(lines=context.lines(),
-                                                                 before=context.lines_before(10)))
-            logger.exception(e)
-            raise e 
+        except strictyaml.exceptions.YAMLValidationError as og_e:
+            try: 
+                conf_fh.seek(0)
+                config = strictyaml.load(conf_fh.read(), fallbackSchema) 
+            except strictyaml.exceptions.YAMLValidationError as e:
+                context = og_e.args[2]
+                logger.error(
+                    "Error in Config:\n{before}\n"
+                    " #vvvv Error Line vvvv#{lines}".format(
+                        lines=context.lines(),
+                        before=context.lines_before(10)))
+                logger.exception(og_e)
+                raise og_e 
         sample_conf_path=config["samples"].data
     # Check user provided
     if args.conf is not None:
@@ -1146,6 +1221,9 @@ def augment_config_with_args(args, config):
     config["var_filter_mqm"] = args.var_filter_mqm
     config["var_filter_sap"] = args.var_filter_sap
     config["var_filter_qual"] = args.var_filter_qual
+    
+    config["read_filter_len"] = args.read_filter_len
+    config["read_filter_qual"] = args.read_filter_qual
     config["cns_min_cov"] = args.cns_min_cov
     config["cns_gt_adjust"] = args.cns_gt_adjust
     if args.var_special_interest:
@@ -1532,10 +1610,13 @@ def prep_snake_call(args, rerun_dict, snake_cmd, name="Change"):
             outs, err = prep_proc.communicate()
             ret = prep_proc.wait()
             if ret!=0:
-                handle_snakemake_syntax_error(
-                        "{err}\n{out}".format(err=mod_string(err),
-                                              out=mod_string(outs)),
-                                              args.blame)
+                try:
+                    handle_snakemake_syntax_error(
+                            "{err}\n{out}".format(err=mod_string(err),
+                                                  out=mod_string(outs)),
+                                                  args.blame)
+                except AttributeError:
+                    pass
                 raise SNAKEMAKE_RUN_ERROR(ret, call=' '.join(snake_cmd),
                                           stdout=mod_string(outs),
                                           stderr=mod_string(err))
